// convert all schedule .xls and .ods files in current folder to .xml files
// 

Number.prototype.FormatAsString = function(width) 
  { 
  var ret = this.toString();

  while(ret.length < width)
    ret = "0" + ret;

  return ret;
  }

function GetDuration(rDate1, rDate2)
  {
  var nDuration = ((rDate2.getHours() - rDate1.getHours()) * 3600) + ((rDate2.getMinutes() - rDate1.getMinutes()) * 60) + (rDate2.getSeconds() - rDate1.getSeconds());

  if(nDuration < 0)
    nDuration = 86400 + nDuration;

  return nDuration;
  }

function StringMapToIndex(rMap, sKey, sKeyName, sValueName)
    {
    for(var i=0; i < rMap.length; i++)
        {
        if(rMap[i][sKeyName] == sKey)
            return rMap[i][sValueName];
        }

    return -1;
    }

function ImportExcelSchedule(sPath, sFileName)
  {
  var lRet = 0;

  // imported events

  var nTotal = 0;
  var rEvents = new Array();

  var fFileSystem = new ActiveXObject("Scripting.FileSystemObject"); 

  // start Excel and open file

  try
    {
    var xlApp = new ActiveXObject("Excel.Application"); 
    var xlWorkbook = xlApp.Workbooks.Open(sPath + sFileName);

    // get parameters

    var nModuleNo = xlWorkbook.ActiveSheet.Cells(3,2).Value;
    var nProgramNo = xlWorkbook.ActiveSheet.Cells(4,2).Value;
    var rDate = new Date(xlWorkbook.ActiveSheet.Cells(6,2).Value);

    rDate.setHours(0, 0, 0, 0);

    // process events

    for(i=9; i <= 1000; i++)
      {
      var nTime = (xlWorkbook.ActiveSheet.Cells(i,1).Value * 24 * 60);

      if(isNaN(nTime))
        break;

      rEvents[nTotal] = new Array(10);

      rDate.setHours(parseInt(nTime / 60), nTime % 60, 0, 0);
      rEvents[nTotal]["start"] = new Date(rDate);

      if(nTotal)
        rEvents[nTotal-1]["dur"] = GetDuration(rEvents[nTotal-1]["start"], rEvents[nTotal]["start"]);
      else
        rEvents[nTotal]["dur"] = 0;

      rEvents[nTotal]["name"] = xlWorkbook.ActiveSheet.Cells(i,2).Value;
      rEvents[nTotal]["desc"] = xlWorkbook.ActiveSheet.Cells(i,3).Value;

      rEvents[nTotal]["cat"] = StringMapToIndex([
		{ "code": 0,  "txt": "" },
		{ "code": 1,  "txt": "" },
		{ "code": 2,  "txt": "" },
		{ "code": 3,  "txt": "" },
		{ "code": 4,  "txt": "" },
		{ "code": 5,  "txt": " " },
		{ "code": 6,  "txt": "" },
		{ "code": 7,  "txt": "" },
		{ "code": 8,  "txt": "/" },
		{ "code": 9,  "txt": " " },
		{ "code": 10, "txt": "/" }
		], xlWorkbook.ActiveSheet.Cells(i,4).Value, "txt", "code");

      var nAge = xlWorkbook.ActiveSheet.Cells(i,5).Value;
      rEvents[nTotal]["age"] = isNaN(nAge) ? 0 : nAge;

      nTotal++;
      }

    if(nTotal)
      {
      var nTime = (xlWorkbook.ActiveSheet.Cells(6,4).Value * 24 * 60);

      if(isNaN(nTime))
        rDate.setHours(0, 0, 0, 0);
      else
        rDate.setHours(parseInt(nTime / 60), nTime % 60, 0, 0);

      rEvents[nTotal-1]["dur"] = GetDuration(rEvents[nTotal-1]["start"], rDate);
      }

    // close file in Excel

    xlWorkbook.Close();

    // delete file

    fFileSystem.DeleteFile(sPath + sFileName);
    }
  catch(e)
    {
    // exception

    lRet = 1;
    }
  finally
    {
    if(xlApp)
      xlApp.Quit();
    }

  // export events to XML file

  try
    {
    if(nTotal)
      {
      // open output xml file

      var sOutputFileName = sPath + "epg_" + ((nModuleNo < 10) ? "0" : "") + nModuleNo + "_" + ((nProgramNo < 10) ? "0" : "") + nProgramNo + ".xml";

      var fOutFile;

      if(fFileSystem.FileExists(sOutputFileName))
        {
        // read existing file and delete end </tv> tag

        var fExFile = fFileSystem.OpenTextFile(sOutputFileName, 1);
        var sContent = "";

        while(!fExFile.AtEndOfStream)
          {
          var str = fExFile.ReadLine();

          if(str.search("</tv>") != -1)
            break;

          sContent += str;
          sContent += "\n";
          }

        fExFile.Close();

        // write all except end </tv> tag to new file

        fOutFile = fFileSystem.CreateTextFile(sOutputFileName, true);

        fOutFile.WriteLine(sContent);
        }
      else
        {
        // create new file

        fOutFile = fFileSystem.CreateTextFile(sOutputFileName, true);

        // add header

        fOutFile.WriteLine("<?xml version=\"1.0\" encoding=\"Win-1251\"?>");
        fOutFile.WriteLine("<tv>");
        }

      for(i=0; i < nTotal; i++)
        {
        var rEnd = new Date(rEvents[i]["start"]);
        rEnd.setSeconds(rEvents[i]["dur"]);

        fOutFile.WriteLine("<programme start=\"" + 
          rEvents[i]["start"].getFullYear().FormatAsString(4) + 
          (rEvents[i]["start"].getMonth()+1).FormatAsString(2) + 
          rEvents[i]["start"].getDate().FormatAsString(2) + 
          rEvents[i]["start"].getHours().FormatAsString(2) + 
          rEvents[i]["start"].getMinutes().FormatAsString(2) + "00 +0000\" stop=\"" + 
          rEnd.getFullYear().FormatAsString(4) + 
          (rEnd.getMonth()+1).FormatAsString(2) + 
          rEnd.getDate().FormatAsString(2) + 
          rEnd.getHours().FormatAsString(2) + 
          rEnd.getMinutes().FormatAsString(2) + "00 +0000\">");

        if(rEvents[i]["name"] && (rEvents[i]["name"].length > 0))
          fOutFile.WriteLine("  <title>" + rEvents[i]["name"] + "</title>");

        if(rEvents[i]["desc"] && (rEvents[i]["desc"].length > 0))
          fOutFile.WriteLine("  <desc>" + rEvents[i]["desc"] + "</desc>");

        if(rEvents[i]["cat"] > 0)
          fOutFile.WriteLine("  <category>" + rEvents[i]["cat"] + "</category>");

        if(rEvents[i]["age"] > 0)
          fOutFile.WriteLine("  <rating>" + rEvents[i]["age"] + "</rating>");

        fOutFile.WriteLine("</programme>");
        }

      fOutFile.WriteLine("</tv>");

      fOutFile.Close();
      }
    }
  catch(e)
    {
    // exception

    lRet = 2;
    }

  return lRet;
  }

function ImportOOCalcSchedule(sPath, sFileName)
  {
  var lRet = 0;

  // imported events

  var nTotal = 0;
  var rEvents = new Array();

  var fFileSystem = new ActiveXObject("Scripting.FileSystemObject"); 

  // start OpenOffice Calc and open file

  try
    {
    var ooServiceManager = new ActiveXObject("com.sun.star.ServiceManager");
    var ooDesktop = ooServiceManager.createInstance("com.sun.star.frame.Desktop");

    var args = new Array();

    var ooDoc = ooDesktop.loadComponentFromURL("file:///" + sPath + sFileName, "_blank", 0, args);
    var ooSheet = ooDoc.GetSheets().getByIndex(0);

    // get parameters

    var nModuleNo = ooSheet.GetCellByPosition(1,2).getValue();
    var nProgramNo = ooSheet.GetCellByPosition(1,3).getValue();
    var sDate = ooSheet.GetCellByPosition(1,5).getString();
    var rDate = new Date(sDate.substr(6, 4), sDate.substr(3, 2) - 1, sDate.substr(0, 2));

    rDate.setHours(0, 0, 0, 0);

    // process events

    for(i=8; i <= 999; i++)
      {
      var sTime = ooSheet.GetCellByPosition(0,i).getString();

      if(!sTime.length)
        break;

      var nTime = parseInt(((sTime.substr(0, 2) * 60)) + parseInt(sTime.substr(3, 2)));

      rEvents[nTotal] = new Array(10);

      rDate.setHours(parseInt(nTime / 60), nTime % 60, 0, 0);
      rEvents[nTotal]["start"] = new Date(rDate);

      if(nTotal)
        rEvents[nTotal-1]["dur"] = GetDuration(rEvents[nTotal-1]["start"], rEvents[nTotal]["start"]);
      else
        rEvents[nTotal]["dur"] = 0;

      rEvents[nTotal]["name"] = ooSheet.GetCellByPosition(1,i).getString();
      rEvents[nTotal]["desc"] = ooSheet.GetCellByPosition(2,i).getString();

      rEvents[nTotal]["cat"] = StringMapToIndex([
		{ "code": 0,  "txt": "" },
		{ "code": 1,  "txt": "" },
		{ "code": 2,  "txt": "" },
		{ "code": 3,  "txt": "" },
		{ "code": 4,  "txt": "" },
		{ "code": 5,  "txt": " " },
		{ "code": 6,  "txt": "" },
		{ "code": 7,  "txt": "" },
		{ "code": 8,  "txt": "/" },
		{ "code": 9,  "txt": " " },
		{ "code": 10, "txt": "/" }
		], ooSheet.GetCellByPosition(3,i).getString(), "txt", "code");

      var nAge = ooSheet.GetCellByPosition(4,i).getString();
      rEvents[nTotal]["age"] = isNaN(nAge) ? 0 : nAge;

      nTotal++;
      }

    if(nTotal)
      {
      var sTime = ooSheet.GetCellByPosition(3,5).getString();

      if(!sTime.length)
        rDate.setHours(0, 0, 0, 0);
      else
        {
        var nTime = parseInt(((sTime.substr(0, 2) * 60)) + parseInt(sTime.substr(3, 2)));
        rDate.setHours(parseInt(nTime / 60), nTime % 60, 0, 0);
        }

      rEvents[nTotal-1]["dur"] = GetDuration(rEvents[nTotal-1]["start"], rDate);
      }

    // close file

    ooDoc.Close(1);
    ooDesktop.Terminate();

    // delete file

    fFileSystem.DeleteFile(sPath + sFileName);
    }
  catch(e)
    {
    // exception

    lRet = 1;
    }

  // export events to XML file

  try
    {
    if(nTotal)
      {
      // open output xml file

      var sOutputFileName = sPath + "epg_" + ((nModuleNo < 10) ? "0" : "") + nModuleNo + "_" + ((nProgramNo < 10) ? "0" : "") + nProgramNo + ".xml";

      var fOutFile;

      if(fFileSystem.FileExists(sOutputFileName))
        {
        // read existing file and delete end </tv> tag

        var fExFile = fFileSystem.OpenTextFile(sOutputFileName, 1);
        var sContent = "";

        while(!fExFile.AtEndOfStream)
          {
          var str = fExFile.ReadLine();

          if(str.search("</tv>") != -1)
            break;

          sContent += str;
          sContent += "\n";
          }

        fExFile.Close();

        // write all except end </tv> tag to new file

        fOutFile = fFileSystem.CreateTextFile(sOutputFileName, true);

        fOutFile.WriteLine(sContent);
        }
      else
        {
        // create new file

        fOutFile = fFileSystem.CreateTextFile(sOutputFileName, true);

        // add header

        fOutFile.WriteLine("<?xml version=\"1.0\" encoding=\"Win-1251\"?>");
        fOutFile.WriteLine("<tv>");
        }

      for(i=0; i < nTotal; i++)
        {
        var rEnd = new Date(rEvents[i]["start"]);
        rEnd.setSeconds(rEvents[i]["dur"]);

        fOutFile.WriteLine("<programme start=\"" + 
          rEvents[i]["start"].getFullYear().FormatAsString(4) + 
          (rEvents[i]["start"].getMonth()+1).FormatAsString(2) + 
          rEvents[i]["start"].getDate().FormatAsString(2) + 
          rEvents[i]["start"].getHours().FormatAsString(2) + 
          rEvents[i]["start"].getMinutes().FormatAsString(2) + "00 +0000\" stop=\"" + 
          rEnd.getFullYear().FormatAsString(4) + 
          (rEnd.getMonth()+1).FormatAsString(2) + 
          rEnd.getDate().FormatAsString(2) + 
          rEnd.getHours().FormatAsString(2) + 
          rEnd.getMinutes().FormatAsString(2) + "00 +0000\">");

        if(rEvents[i]["name"] && (rEvents[i]["name"].length > 0))
          fOutFile.WriteLine("  <title>" + rEvents[i]["name"] + "</title>");

        if(rEvents[i]["desc"] && (rEvents[i]["desc"].length > 0))
          fOutFile.WriteLine("  <desc>" + rEvents[i]["desc"] + "</desc>");

        if(rEvents[i]["cat"] > 0)
          fOutFile.WriteLine("  <category>" + rEvents[i]["cat"] + "</category>");

        if(rEvents[i]["age"] > 0)
          fOutFile.WriteLine("  <rating>" + rEvents[i]["age"] + "</rating>");

        fOutFile.WriteLine("</programme>");
        }

      fOutFile.WriteLine("</tv>");

      fOutFile.Close();
      }
    }
  catch(e)
    {
    // exception

    lRet = 2;
    }

  return lRet;
  }

function main()
  {
  // list files

  var fSO = new ActiveXObject("Scripting.FileSystemObject");

  var fFolder = fSO.GetFolder(".\\");

  var fEnumerator = new Enumerator(fFolder.files);

  for(; !fEnumerator.atEnd(); fEnumerator.moveNext())
    {
    var sFileName = fEnumerator.item().Name;
    var sExt = sFileName.substr(sFileName.length-3, 3).toLowerCase();

    if(sExt == "xls")
      ImportExcelSchedule(fFolder.Path + "\\", sFileName);
    }

  fFolder = fSO.GetFolder(".\\");
  fEnumerator = new Enumerator(fFolder.files);

  for(; !fEnumerator.atEnd(); fEnumerator.moveNext())
    {
    var sFileName = fEnumerator.item().Name;
    var sExt = sFileName.substr(sFileName.length-3, 3).toLowerCase();

    if(sExt == "ods")
      ImportOOCalcSchedule(fFolder.Path + "\\", sFileName);
    }
  }

main();
